package com.free.config;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

/**
 * 自定义异常拦截
 */
@ControllerAdvice
public class GlobalExceptionController {
  private static final Logger log = LoggerFactory.getLogger(GlobalExceptionController.class);

  @ExceptionHandler(value = CustomizationException.class)
  @ResponseBody
  public ResponseFormat exception(HttpServletRequest req, CustomizationException ce) {
    log.error("==========自定义异常捕获==========");
    log.error(getExceptionInfo(ce));
    return ResponseFormat.customizationError(ce);
  }

  @ExceptionHandler({ MethodArgumentNotValidException.class })
  @ResponseBody
  public ResponseFormat methodArgNotValidException(MethodArgumentNotValidException ex) {
    BindingResult bindingResult = ex.getBindingResult();
    StringBuilder sb = new StringBuilder("校验失败:");
    char appendChar = ',';
    for (FieldError fieldError : bindingResult.getFieldErrors()) {
      sb.append(fieldError.getField()).append("：").append(fieldError.getDefaultMessage()).append(appendChar);
    }
    String msg = sb.toString();
    msg = StringUtils.trimTrailingCharacter(msg, appendChar);
    System.out.println(msg);
    ResponseFormat rf = ResponseFormat.validError();
    rf.setErrmsg(msg);
    return rf;
  }

  @ExceptionHandler(value = Exception.class)
  @ResponseBody
  public ResponseFormat exceptionHandler(HttpServletRequest req, Exception e) {
    log.error("==========全局异常捕获==========");
    log.error(getExceptionInfo(e));
    return ResponseFormat.error();
  }

  private static String getExceptionInfo(Exception ex) {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    PrintStream printStream = new PrintStream(out);
    ex.printStackTrace(printStream);
    String rs = new String(out.toByteArray());
    try {
      printStream.close();
      out.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
    return rs;
  }

}
