package com.free.config;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

/**
 * 格式化返回结果
 */
@Data
@Schema(description = "响应返回数据对象")
public class ResponseFormat {
  private String errcode;
  private String errmsg;
  private Object data;

  /**
   * 接口返回成功
   * 
   * @param data
   * @return
   */
  public static ResponseFormat success(Object data) {
    ResponseFormat rf = new ResponseFormat();
    rf.setData(data);
    rf.setErrcode("0");
    rf.setErrmsg("ok");
    return rf;
  }

  public static ResponseFormat success() {
    ResponseFormat rf = new ResponseFormat();
    rf.setErrcode("0");
    rf.setErrmsg("ok");
    return rf;
  }

  /**
   * 接口返回自定义异常
   * 
   * @param e 自定义异常
   * @return
   */
  public static ResponseFormat customizationError(CustomizationException e) {
    ResponseFormat rf = new ResponseFormat();
    rf.setErrmsg(e.getErrMsg());
    rf.setErrcode(e.getErrCode());
    return rf;
  }

  /**
   * 参数校验失败
   * @return
   */
  public static ResponseFormat validError() {
    ExceptionEnum serviceFault = ExceptionEnum.VALID;
    ResponseFormat rf = new ResponseFormat();
    rf.setErrcode(serviceFault.getResultCode());
    return rf;
  }

  /**
   * 接口返回其他异常
   * 
   * @return
   */
  public static ResponseFormat error() {
    ExceptionEnum serviceFault = ExceptionEnum.SERVICE_FAULT;
    ResponseFormat rf = new ResponseFormat();
    rf.setErrcode(serviceFault.getResultCode());
    rf.setErrmsg(serviceFault.getResultMsg());
    return rf;
  }
}
