package com.free.controller;

import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.free.config.ResponseFormat;
import com.free.service.MessageService;
import com.free.service.QuestionService;
import com.free.service.TransferApplyService;
import com.free.service.UserService;
import com.free.utils.JwtUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;

@RestController
@RequestMapping("/index")
@Api(tags = "首页统计")
public class IndexController {

  @Autowired
  private TransferApplyService transferApplyService;
  @Autowired
  private QuestionService questionService;
  @Autowired
  private UserService userService;
  @Autowired
  private MessageService messageService;

  @ApiOperation("首页统计接口")
  @GetMapping("")
  public Object index() {
    Map returnData = new HashMap<>();
    // 1.转人工申请待处理数量(不区分用户)
    QueryWrapper qw1 = new QueryWrapper<>();
    qw1.eq("is_agree", TransferApplyController.TransferApplyNotAgree);
    Long num1 = transferApplyService.count(qw1);
    returnData.put("n1", num1);
    // 2.转人工申请未结束数量(区分用户,管理员查所有)
    QueryWrapper qw2 = new QueryWrapper<>();
    qw2.eq("is_agree", TransferApplyController.TransferApplyAgree);
    qw2.eq("is_close", TransferApplyController.TransferApplyNotClose);
    Map userInfo = JwtUtil.getDetails(null);
    String type = (String) userInfo.get("type");
    if (!LoginController.adminLoginType.equals(type)) {
      String customer_id = (String) userInfo.get("id");
      qw2.eq("customer_id", customer_id);
    }
    Long num2 = transferApplyService.count(qw2);
    returnData.put("n2", num2);
    // 3.题库数量
    Long num3 = questionService.count();
    returnData.put("n3", num3);
    // 4.用户数量
    Long num4 = userService.count();
    returnData.put("n4", num4);
    // 5.未回复留言数量
    QueryWrapper msgQw = new QueryWrapper<>();
    msgQw.eq("is_reply", messageService.not_reply_sign);
    Long num5 = messageService.count(msgQw);
    returnData.put("n5", num5);
    return ResponseFormat.success(returnData);
  }
}
