package com.free.controller;

import java.util.List;
import java.util.Map;
import java.time.LocalDateTime;
import java.util.HashMap;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.free.config.CustomizationException;
import com.free.config.ExceptionEnum;
import com.free.config.ResponseFormat;
import com.free.dto.TransferApplyCreateDTO;
import com.free.entity.TransferApply;
import com.free.mq.MqService;
import com.free.service.TransferApplyService;
import com.free.utils.JwtUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@RequestMapping("/ta")
@Api(tags = "转人工申请")
public class TransferApplyController {
  @Autowired
  private TransferApplyService service;
  @Autowired
  private MqService mqService;
  /** 转人工申请,受理转人工-受理申请标识 */
  public final static String TransferApplyAgree = "0";
  /** 转人工申请,受理转人工-未受理标识 */
  public final static String TransferApplyNotAgree = "1";
  /** 转人工申请,受理转人工-结束标识 */
  public final static int TransferApplyClose = 0;
  /** 转人工申请,受理转人工-未结束标识 */
  public final static int TransferApplyNotClose = 1;

  @ApiOperation("结束转人工服务")
  @PostMapping("/close/{id}")
  public Object close(@PathVariable String id) {
    TransferApply data = service.getById(id);
    if (null == data) {
      throw new CustomizationException(ExceptionEnum.NOT_FOUND);
    }
    data.setIs_close(TransferApplyClose);
    service.updateById(data);
    return ResponseFormat.success();
  }

  /** 创建数据 */
  @ApiOperation("创建数据")
  @PostMapping("")
  public Object save(@RequestBody @Valid TransferApplyCreateDTO body) {
    // 查询是否有申请但是未受理的数据
    QueryWrapper checkHasAppplyNotAgreeQw = new QueryWrapper<>();
    checkHasAppplyNotAgreeQw.eq("user_id", body.getUser_id());
    checkHasAppplyNotAgreeQw.eq("is_agree", TransferApplyNotAgree);
    Map hasApplyNotAgree = service.getMap(checkHasAppplyNotAgreeQw);
    if (null != hasApplyNotAgree) {
      // 抛出异常并发送mq提醒客服人员
      mqService.sendMsgToAdminForApplyCreate(hasApplyNotAgree);
      throw new CustomizationException(ExceptionEnum.APPLY_NOT_AGREE);
    }
    // 查询是否有申请但是受理中的数据
    QueryWrapper checkHasApplyNotCloseQw = new QueryWrapper<>();
    checkHasApplyNotCloseQw.eq("user_id", body.getUser_id());
    checkHasApplyNotCloseQw.eq("is_agree", TransferApplyAgree);
    checkHasApplyNotCloseQw.eq("is_close", TransferApplyNotClose);
    Map hasApplyNotClose = service.getMap(checkHasApplyNotCloseQw);
    if (null != hasApplyNotClose) {
      throw new CustomizationException(ExceptionEnum.APPLY_NOT_ClOSE);
    }
    TransferApply data = new TransferApply();
    data.setApply_time(LocalDateTime.now());
    data.setUser_id(body.getUser_id());
    data.setIs_close(TransferApplyNotClose);
    this.service.save(data);
    QueryWrapper qw = new QueryWrapper<>();
    qw.eq("id", data.getId());
    Map returnData = this.service.getMap(qw);
    // 创建完数据后需要通过 mq 给管理员提示
    mqService.sendMsgToAdminForApplyCreate(returnData);
    return ResponseFormat.success(returnData);
    // return ResponseFormat.success();
  }

  /** 修改数据 */
  @ApiOperation("修改数据")
  @PostMapping("/{id}")
  public Object update(@PathVariable String id, @RequestBody TransferApply data) {
    // 从token获取信息, 只有客服人员能处理
    Map userMap = JwtUtil.getDetails(null);
    String type = (String) userMap.get("type");
    if (!LoginController.csLoginType.equals(type)) {
      throw new CustomizationException(ExceptionEnum.ACCPET_USER_TYPE_FAULT);

    }
    // 获取客服人员id
    String customer_id = (String) userMap.get("id");

    // 如果是受理申请,需要查看,除该数据外.当前申请的用户 是否有已被受理 且 未关闭 的转人工申请.
    // 如果有: 则无法受理当前申请, 需要将之前 受理且未关闭的申请先关闭后再受理
    String is_agree = data.getIs_agree();
    if (TransferApplyAgree.equals(is_agree)) {
      TransferApply oldData = this.service.getById(id);
      QueryWrapper checkHasApplyAgreeNotCloseQw = new QueryWrapper<>();
      checkHasApplyAgreeNotCloseQw.ne("id", id);
      checkHasApplyAgreeNotCloseQw.eq("user_id", oldData.getUser_id());
      checkHasApplyAgreeNotCloseQw.eq("is_agree", TransferApplyAgree);
      checkHasApplyAgreeNotCloseQw.eq("is_close", TransferApplyNotClose);
      Long applyAgreeNotCloseNum = service.count(checkHasApplyAgreeNotCloseQw);
      if (applyAgreeNotCloseNum > 0) {
        throw new CustomizationException(ExceptionEnum.NEED_CLOSE_OTHER_APPLYS);
      }
      // 再接着比较,如果当前数据已受理,且受理人不一致,需要抛出异常,不允许第二个客服进行受理
      String od_is_agree = oldData.getIs_agree();
      String od_customer_id = oldData.getCustomer_id();
      if(TransferApplyAgree.equals(od_is_agree)&&!od_customer_id.equals(customer_id)) {
        throw new CustomizationException(ExceptionEnum.ALREADY_HAS_CUSTOMER);
      }

    }

    QueryWrapper qw = new QueryWrapper<>();
    qw.eq("id", id);
    TransferApply oldData = this.service.getOne(qw);
    if (null == oldData) {
      throw new CustomizationException(ExceptionEnum.NOT_FOUND);
    }
    TransferApply updateData = new TransferApply();
    // 修改不会修改会话状态 关闭有单独的方法
    updateData.setIs_close(oldData.getIs_close());
    updateData.setId(id);
    
    //设置客服人员数据
    updateData.setCustomer_id(customer_id);
    updateData.setIs_agree(data.getIs_agree());
    this.service.updateById(updateData);
    TransferApply newData = this.service.getById(id);
    // 如果审核通过了, 则需要给申请人发送mq消息.申请人需要知道是谁跟自己对话
    if (newData.getIs_agree().equals(TransferApplyAgree)) {
      Map map = this.service.getUserAndCustomerNameByTransferApply(newData);
      mqService.sendMsgToApplicant(map);
    }
    return ResponseFormat.success(newData);
  }

  /** 列表查询 */
  @ApiOperation("查询数据列表")
  @SuppressWarnings({ "unchecked" })
  @GetMapping()
  public Object list(@RequestParam Map<String, Object> allParams) {
    Long skip = null, limit = null;
    Map map = new HashMap();
    QueryWrapper qw = new QueryWrapper<>();
    qw.orderByDesc("is_close");
    /** 参数处理处理 */
    for (String key : allParams.keySet()) {
      Object value = allParams.get(key);
      if (key.equals("skip")) {
        skip = Long.valueOf(String.valueOf(value));
      } else if (key.equals("limit")) {
        limit = Long.valueOf(String.valueOf(value));
      } else {
        // 其他为查询条件
        qw.eq(key, value);
      }
    }
    /**
     * 分页处理
     */
    List returnList = null;
    if (null != skip && null != limit) {
      IPage page = new Page<>(skip, limit);
      IPage pageResult = service.page(page, qw);
      List data = pageResult.getRecords();
      returnList = service.getUserAndCustomerName(data);
      long total = pageResult.getTotal();
      map.put("total", total);
    } else {
      List list = service.list(qw);
      returnList = service.getUserAndCustomerName(list);
    }
    // 如果查询条件中有customer_id,客服id,需要查出其最后一条发言数据
    returnList = service.getDataLastRecord(returnList);
    // 查询当前用户的未读数量
    returnList = service.getUserNotReadNum(returnList);
    map.put("data", returnList);

    return ResponseFormat.success(map);
  }

  /** 根据id查询 */
  @ApiOperation("查询数据")
  @GetMapping("/{id}")
  public Object fetch(@PathVariable String id) {
    Object newData = service.getById(id);
    return ResponseFormat.success(newData);
  }

  /** 根据id删除数据 */
  @ApiOperation("删除数据")
  @DeleteMapping("/{id}")
  public Object delete(@PathVariable String id) {
    QueryWrapper qw = new QueryWrapper<>();
    qw.eq("id", id);
    Long num = service.count(qw);
    if (num <= 0) {
      throw new CustomizationException(ExceptionEnum.NOT_FOUND);
    }
    service.removeById(id);
    return ResponseFormat.success();
  }
}
