package com.free.mq;

import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.FanoutExchange;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;

@Configuration
public class MqListeners {
  @Lazy
  @Autowired
  RabbitAdmin rabbitAdmin;
  /** 管理用的队列和交换机,主要是平台统一监控: 1.用户进入智能客服;2.用户使用智能客服;3.用户需要转人工; */
  /** 管理用 - 交换机 */
  public final static String adminExName = "adminEx";
  /** 管理用 - 队列名 */
  public final static String adminQueue = "msg";

  /** 用户用 - 交换机 */
  public final static String userExName = "userEx";
  /** 用户用 - 队列名 */
  public final static String userQueue = "umsg";
  /** 用户用 - 路由匹配 *: 用户唯一标识 */
  public final static String userRouterMatch = "*";

  /** 对话用 - 交换机名 */
  public final static String exName = "chatEx";
  /** 对话用 - 队列名 */
  public final static String queueName = "chat";
  /** 对话用 - 路由匹配 *: 转人工申请id */
  public final static String routerMatch ="*";

  public final static boolean durable = false;
  public final static boolean autoDelete = true;

  // admin
  @Bean()
  public Queue adminQueue() {
    System.out.println("create " + adminQueue + " queue");
    return new Queue(adminQueue, durable);
  }

  @Bean()
  FanoutExchange admiExchange() {
    System.out.println("create " + adminExName + " ex");
    FanoutExchange f = new FanoutExchange(adminExName, durable, autoDelete);
    return f;
  }

  @Bean
  Binding bindingAdminExchangeMessage() {
    System.out.println("bind " + adminExName + " +" + adminQueue);
    return BindingBuilder.bind(adminQueue()).to(admiExchange());
  }

  // user
  @Bean
  public Queue userQueue() {
    System.out.println("create " + userQueue + " queue");
    return new Queue(userQueue, durable);
  }

  @Bean
  TopicExchange userExchange() {
    System.out.println("create " + userExName + " ex");
    return new TopicExchange(userExName, durable, autoDelete);
  }

  @Bean
  Binding bindingUserExchangeMessage() {
    System.out.println("bind " + userExName + " +" + userQueue + " with router key " + userRouterMatch);
    return BindingBuilder.bind(userQueue()).to(userExchange()).with(userRouterMatch);
  }

  // chat
  /** 声明队列 */
  @Bean
  public Queue chatQueue() {
    System.out.println("create " + queueName + " queue");
    return new Queue(queueName, durable);
  }

  /** 声明交换机 */
  @Bean
  TopicExchange exchange() {
    System.out.println("create " + exName + " ex");
    return new TopicExchange(exName, durable, autoDelete);
  }

  /** 队列绑定交换机 */
  @Bean
  Binding bindingExchangeMessage() {
    System.out.println("bind " + exName + " +" + queueName + " with router key " + routerMatch);
    return BindingBuilder.bind(chatQueue()).to(exchange()).with(routerMatch);
  }

  // 创建初始化RabbitAdmin对象
  @Bean
  public RabbitAdmin rabbitAdmin(ConnectionFactory connectionFactory) {
    RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);
    // 只有设置为 true，spring 才会加载 RabbitAdmin 这个类
    rabbitAdmin.setAutoStartup(true);
    return rabbitAdmin;
  }

  // 创建交换机和对列
  @Bean
  public void createExchangeQueue() {
    rabbitAdmin.declareExchange(admiExchange());
    rabbitAdmin.declareExchange(userExchange());
    rabbitAdmin.declareExchange(exchange());

    // rabbitAdmin.declareQueue(adminQueue());
    // rabbitAdmin.declareQueue(userQueue());
    // rabbitAdmin.declareQueue(chatQueue());
  }
}
