package com.free.mq;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import java.util.Map;


import java.util.ArrayList;
import java.util.HashMap;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.free.config.FormatJavaTimeModule;

@Service
public class MqService {
  private static final Logger log = LoggerFactory.getLogger(MqService.class);
  @Value("${spring.rabbitmq.host}")
  private String mqIp;
  /** rabbitmq api接口端口 */
  private String mqApiPort = "15672";
  @Value("${spring.rabbitmq.username}")
  private String mqUser;
  @Value("${spring.rabbitmq.password}")
  private String mqPassword;
  /** mq 消费者查询api uri */
  private String mqConsumersUri = "/api/consumers";
  @Autowired
  RabbitTemplate rabbitTemplate;
  /**
   * 获取mq消费者id
   * @return
   * @throws JsonMappingException
   * @throws JsonProcessingException
   */
  public ArrayList getConsumersIds() throws JsonMappingException, JsonProcessingException {
    RestTemplateBuilder builder = new RestTemplateBuilder();
    RestTemplate rt = builder.basicAuthentication(this.mqUser, this.mqPassword).build();
    String url = "http://" + this.mqIp + ":" + this.mqApiPort + this.mqConsumersUri;
    String res = rt.getForObject(url, String.class);
    ObjectMapper om = new ObjectMapper();
    ArrayList<HashMap<String, Object>> list = om.readValue(res, ArrayList.class);
    // 重新整理数据
    ArrayList newList = new ArrayList<>();
    for (HashMap<String, Object> map : list) {
      Map midMap = new HashMap();
      for (Map.Entry<String, Object> hashMap : map.entrySet()) {
        midMap.put(hashMap.getKey(), hashMap.getValue());
      }
      // 只取出 consumer_tag进行分割,取出id
      String consumer_tag = (String) midMap.get("consumer_tag");
      consumer_tag = consumer_tag.replace("T_", "");
      newList.add(consumer_tag);
    }
    return newList;
  }

  /**
   * 发送mq消息
   * 
   * @param ex    交换机名
   * @param queue 队列名
   * @param msg   消息内容
   * @throws JsonProcessingException JSON处理异常
   */
  public void sendMsg(String ex, String queue, Map msg) throws JsonProcessingException {
    System.out.println(msg);
    String strMsg = null;
    ObjectMapper mapper = new ObjectMapper();
    // 空值放过
    mapper.setSerializationInclusion(Include.NON_NULL);
    // LocalDateTime处理
    JavaTimeModule jtmf = FormatJavaTimeModule.init();
    mapper.registerModule(jtmf);
    strMsg = mapper.writeValueAsString(msg);
    rabbitTemplate.convertAndSend(ex, queue, strMsg);
  }

  /**
   * 转人工对话mq消息
   * 
   * @param data ChatRecord数据
   */
  public void sendChatToApply(Map data) {
    Map map = new HashMap<>();
    map.put("type", "chat");
    map.put("data", data);
    String apply_id = (String) data.get("apply_id");
    String queue = MqListeners.queueName + "." + apply_id;
    try {
      this.sendMsg(MqListeners.exName, queue, map);
    } catch (JsonProcessingException e) {
      e.printStackTrace();
      log.error("转人工对话通知mq发送失败", e);
    }
  }

  /**
   * 向管理员订阅队列发送消息,通知管理员去处理转人工申请
   * 
   * @param data 转人工申请数据
   */
  public void sendMsgToAdminForApplyCreate(Map data) {
    String ex = MqListeners.adminExName;
    String queue = MqListeners.adminQueue;
    Map map = new HashMap<>();
    map.put("data", data);
    map.put("type", "notice");
    try {
      this.sendMsg(ex, queue, map);
    } catch (JsonProcessingException e) {
      e.printStackTrace();
      log.error("转人工申请通知mq发送失败", e);
    }
  }

  /**
   * 转人工已受理通知
   * 
   * @param apply
   */
  public void sendMsgToApplicant(Map apply) {
    String ex = MqListeners.userExName;
    String routerKey = MqListeners.userQueue + "." + (String) apply.get("user_id");
    Map map = new HashMap<>();
//    Map data = Utils.objectToMap(apply);
    map.put("data", apply);
    map.put("type", "notice");
    try {
      this.sendMsg(ex, routerKey, map);
    } catch (JsonProcessingException e) {
      e.printStackTrace();
      log.error("转人工申请已通过通知,mq发送失败", e);
    }
  }
}
