package com.free.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.free.controller.ChatRecordController;
import com.free.entity.TransferApply;
import com.free.entity.User;
import com.free.entity.system.Customer;
import com.free.mapper.TransferApplyMapper;
import com.free.service.system.CustomerService;
import com.free.utils.JwtUtil;
import com.free.utils.Utils;

@Service
public class TransferApplyService extends ServiceImpl<TransferApplyMapper, TransferApply> {
  @Autowired
  private CustomerService customerService;
  @Autowired
  private ChatRecordService chatRecordService;
  @Autowired
  private UserService userService;

  // @Autowired
  // private UserService userService
  /**
   * 将列表中的客服id和用户io的数据取出来
   * 
   * @param list
   * @return
   */
  public List<Map> getUserAndCustomerName(List<TransferApply> list) {
    List<Map> returnData = new ArrayList<>();
    for (TransferApply i : list) {
      Map map = this.getUserAndCustomerNameByTransferApply(i);
      returnData.add(map);

    }
    return returnData;
  }

  public Map getUserAndCustomerNameByTransferApply(TransferApply i) {
    Map map = Utils.objectToMap(i);
    String customer_id = i.getCustomer_id();
    if (null != customer_id) {
      Customer cData = customerService.getById(customer_id);
      if (null != cData) {
        map.put("customer_name", cData.getNick_name());
      }
    }
    String user_id = i.getUser_id();
    if (null != user_id) {
      User uData = userService.getById(user_id);
      if (null != uData) {
        // 查询用户名称
        map.put("user_name", uData.getName());
      }
    }
    return map;
  }

  /**
   * 查询当前用户查出的列表中的未读数量
   * 
   * @param list
   * @return
   */
  public List<Map> getUserNotReadNum(List<Map> list) {
    String token = null;
    try {
      token = JwtUtil.getToken();
    } catch (Exception e) {
    }
    if (null == token) {
      return list;
    }
    Map userInfo = JwtUtil.getDetails(null);
    String user_id = (String) userInfo.get("id");
    for (Map map : list) {
      String apply_id = (String) map.get("id");
      QueryWrapper qw = new QueryWrapper<>();
      qw.eq("apply_id", apply_id);
      qw.eq("is_read", ChatRecordController.chatRecordNotReadSign);
      qw.ne("speaker", user_id);
      Long num = this.chatRecordService.count(qw);
      map.put("notRead", num);
    }
    return list;
  }

  /**
   * 获取每个转人工申请中聊天记录的最后一条
   * 
   * @param list
   * @return
   */
  public List<Map> getDataLastRecord(List<Map> list) {
    for (Map map : list) {
      String apply_id = (String) map.get("id");
      QueryWrapper qw = new QueryWrapper<>();
      qw.select("content", "time", "is_read", "content_type");
      qw.eq("apply_id", apply_id);
      qw.orderByDesc("time");
      Map chatRecord = chatRecordService.getMap(qw);
      map.put("chatRecord", chatRecord);
    }

    return list;
  }
}
