package com.free.service.system;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.free.entity.system.Menus;
import com.free.mapper.system.MenusMapper;
import com.free.utils.Utils;

@Service
public class MenusService extends ServiceImpl<MenusMapper, Menus> {

  public void initData(List<Map> list) {
    // 清空表
    this.baseMapper.truncate();
    this.toInsertMenusData(list, null);
  }

  public void toInsertMenusData(List<Map> list, String parent_id) {
    ObjectMapper mapper = new ObjectMapper();
    for (int i = 0; i < list.size(); i++) {
      Map m = list.get(i);
      // 获取children,然后把map里的children删了,要不影响之后转换成Menus
      List<Map> children = (List<Map>) m.get("children");
      m.remove("children");
      // 获取排序,没有顺序就填充
      Integer on = (Integer) m.get("order_num");
      Long order_num = null;
      if (null == on) {
        order_num = i + 1L;
      } else {
        order_num = Long.valueOf(on);
      }
      // 如果有parent_id,说明是子目录,需要加上
      m.put("parent_id", parent_id);
      // 转船成menus对象
      Menus menuData = mapper.convertValue(m, Menus.class);
      menuData.setOrder_num(order_num);
      // 创建数据,并获得当前id
      this.save(menuData);
      if (null == children) {
        continue;
      }
      // 处理children,获取parent_id
      String children_parent_id = menuData.getId();
      toInsertMenusData(children, children_parent_id);
    }
  }

  /**
   * 递归查询所有菜单
   * 
   * @return
   */
  public List<Map<String, Object>> recursionMenus() {
    List<Menus> list = this.list();
    // 转换成Map-List,因为之后要增加children属性.Menus不能直接加
    List<Map<String, Object>> allMapList = this.menusListTurnToMenusListMaps(list);
    // 1.取出第一层级,随后开始递归
    List<Map<String, Object>> firstLevelList = this.getFirstLevelList(allMapList);
    List<Map<String, Object>> returnList = this.treeMenu(firstLevelList, allMapList);
    return returnList;
  }

  /**
   * 根据传来的所有目录,递归处理
   * 
   * @param list
   * @return
   */
  public List<Map<String, Object>> recursionMenus(List<Menus> list) {
    // 转换成Map-List,因为之后要增加children属性.Menus不能直接加
    List<Map<String, Object>> allMapList = this.menusListTurnToMenusListMaps(list);
    // 1.取出第一层级,随后开始递归
    List<Map<String, Object>> firstLevelList = this.getFirstLevelList(allMapList);
    List<Map<String, Object>> returnList = this.treeMenu(firstLevelList, allMapList);
    return returnList;
  }

  /**
   * 将List转为List<Map>
   * 
   * @param list
   * @return
   */
  public List<Map<String, Object>> menusListTurnToMenusListMaps(List<Menus> list) {
    List<Map<String, Object>> allMapList = new ArrayList<>();
    for (Menus m : list) {
      Map<String, Object> midMap = Utils.objectToMap(m);
      allMapList.add(midMap);
    }
    return allMapList;
  }

  /**
   * 用List<Map>类型的数据,过滤出第一层级数据
   * 
   * @param list
   * @return
   */
  public List<Map<String, Object>> getFirstLevelList(List<Map<String, Object>> list) {
    List<Map<String, Object>> firstLevelList = list.stream().filter(f -> null == f.get("parent_id"))
        .collect(Collectors.toList());
    return firstLevelList;
  }

  /**
   * 递归,第一层级开始,递归找子目录
   * 
   * @param list    第一层级列表
   * @param allList 所有目录
   * @return
   */
  public List<Map<String, Object>> treeMenu(List<Map<String, Object>> list, List<Map<String, Object>> allList) {
    for (Map<String, Object> m : list) {
      // 取id,找parent_id 与其一致的数据
      String id = (String) m.get("id");
      String name = (String) m.get("name");
      List<Map<String, Object>> children = new ArrayList<>();
      // 找children
      for (Map<String, Object> am : allList) {
        String parent_id = (String) am.get("parent_id");
        if (null != parent_id && parent_id.equals(id)) {
          // 为子目录,放入children中
          am.put("parent_name", name);
          children.add(am);
        }
      }
      // children中没有数据,直接跳过
      if (children.size() <= 0) {
        continue;
      }
      // children中有数据,需要进入children中找子目录
      children = this.treeMenu(children, allList);
      m.put("children", children);
    }
    return list;
  }

}
